package betterquesting.api2.client.gui.panels.lists;

import betterquesting.api2.client.gui.misc.ComparatorGuiDepth;
import betterquesting.api2.client.gui.misc.GuiRectangle;
import betterquesting.api2.client.gui.misc.IGuiRect;
import betterquesting.api2.client.gui.panels.IGuiPanel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

/* loaded from: betterquesting_api2_client_gui_panels_lists.jar:betterquesting/api2/client/gui/panels/lists/CanvasCullingManager.class */
public class CanvasCullingManager {
    private final List<IGuiPanel> dynamicPanels;
    private final Map<String, RegionInfo> panelRegions;
    private final List<IGuiPanel> cachedPanels;
    private final int gridSize;

    public CanvasCullingManager() {
        this(128);
    }

    public CanvasCullingManager(int gridSize) {
        this.dynamicPanels = new ArrayList();
        this.panelRegions = new HashMap();
        this.cachedPanels = new CopyOnWriteArrayList();
        this.gridSize = gridSize;
    }

    public void reset() {
        this.dynamicPanels.clear();
        this.panelRegions.clear();
        this.cachedPanels.clear();
    }

    public void addPanel(IGuiPanel panel, boolean useBlocks) {
        if (!useBlocks) {
            if (!this.dynamicPanels.contains(panel)) {
                this.dynamicPanels.add(panel);
                this.cachedPanels.add(panel);
                return;
            }
            return;
        }
        int x = panel.getTransform().getX();
        int y = panel.getTransform().getY();
        int x2 = (x - (((x % this.gridSize) + this.gridSize) % this.gridSize)) / this.gridSize;
        int y2 = (y - (((y % this.gridSize) + this.gridSize) % this.gridSize)) / this.gridSize;
        RegionInfo cbl = this.panelRegions.get(x2 + "," + y2);
        if (cbl == null) {
            RegionInfo cbl2 = new RegionInfo(x2, y2, this.gridSize);
            int minX = Math.min(cbl2.rect.x, panel.getTransform().getX());
            int minY = Math.min(cbl2.rect.y, panel.getTransform().getY());
            int maxX = Math.max(cbl2.rect.x + cbl2.rect.w, panel.getTransform().getX() + panel.getTransform().getWidth());
            int maxY = Math.max(cbl2.rect.y + cbl2.rect.h, panel.getTransform().getY() + panel.getTransform().getHeight());
            cbl2.rect.x = minX;
            cbl2.rect.y = minY;
            cbl2.rect.w = maxX - minX;
            cbl2.rect.h = maxY - minY;
            cbl2.panels.add(panel);
            this.panelRegions.put(x2 + "," + y2, cbl2);
            return;
        }
        if (cbl.panels.contains(panel)) {
            return;
        }
        cbl.panels.add(panel);
        int minX2 = Math.min(cbl.rect.x, panel.getTransform().getX());
        int minY2 = Math.min(cbl.rect.y, panel.getTransform().getY());
        int maxX2 = Math.max(cbl.rect.x + cbl.rect.w, panel.getTransform().getX() + panel.getTransform().getWidth());
        int maxY2 = Math.max(cbl.rect.y + cbl.rect.h, panel.getTransform().getY() + panel.getTransform().getHeight());
        cbl.rect.x = minX2;
        cbl.rect.y = minY2;
        cbl.rect.w = maxX2 - minX2;
        cbl.rect.h = maxY2 - minY2;
        if (cbl.enabled) {
            this.cachedPanels.add(panel);
            this.cachedPanels.sort(ComparatorGuiDepth.INSTANCE);
        }
    }

    public void removePanel(IGuiPanel panel) {
        if (!this.dynamicPanels.remove(panel)) {
            for (RegionInfo cbl : this.panelRegions.values()) {
                if (cbl.panels.remove(panel)) {
                    cbl.refreshBounds();
                    this.cachedPanels.remove(panel);
                    return;
                }
            }
            return;
        }
        this.cachedPanels.remove(panel);
    }

    public List<IGuiPanel> getVisiblePanels() {
        return this.cachedPanels;
    }

    public void updateVisiblePanels(IGuiRect region) {
        boolean changed = false;
        for (RegionInfo cb : this.panelRegions.values()) {
            boolean prevState = cb.enabled;
            cb.enabled = overlapCheck(cb.rect, region);
            if (prevState != cb.enabled) {
                if (cb.enabled) {
                    this.cachedPanels.addAll(cb.panels);
                } else {
                    this.cachedPanels.removeAll(cb.panels);
                }
                changed = true;
            }
        }
        if (changed) {
            this.cachedPanels.sort(ComparatorGuiDepth.INSTANCE);
        }
    }

    private static boolean overlapCheck(IGuiRect rect1, IGuiRect rect2) {
        return rect1.getX() + rect1.getWidth() >= rect2.getX() && rect1.getX() <= rect2.getX() + rect2.getWidth() && rect1.getY() + rect1.getHeight() >= rect2.getY() && rect1.getY() <= rect2.getY() + rect2.getHeight();
    }

    /* loaded from: betterquesting_api2_client_gui_panels_lists.jar:betterquesting/api2/client/gui/panels/lists/CanvasCullingManager$RegionInfo.class */
    private static class RegionInfo {
        private final List<IGuiPanel> panels;
        private boolean enabled;
        private final GuiRectangle rect;

        private RegionInfo(int blockX, int blockY, int gridSize) {
            this.panels = new ArrayList();
            this.enabled = false;
            this.rect = new GuiRectangle(blockX * gridSize, blockY * gridSize, gridSize, gridSize);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public void refreshBounds() {
            boolean set = false;
            int minX = 0;
            int minY = 0;
            int maxX = 0;
            int maxY = 0;
            for (IGuiPanel pan : this.panels) {
                if (!set) {
                    set = true;
                    minX = pan.getTransform().getX();
                    minY = pan.getTransform().getY();
                    maxX = minX + pan.getTransform().getWidth();
                    maxY = minY + pan.getTransform().getHeight();
                } else {
                    minX = Math.min(minX, pan.getTransform().getX());
                    minY = Math.min(minY, pan.getTransform().getY());
                    maxX = Math.max(maxX, pan.getTransform().getX() + pan.getTransform().getWidth());
                    maxY = Math.max(maxY, pan.getTransform().getY() + pan.getTransform().getHeight());
                }
            }
            this.rect.x = minX;
            this.rect.y = minY;
            this.rect.w = maxX - minX;
            this.rect.h = maxY - minY;
        }
    }
}
